## TimeTracker.py
from AccessControl import ClassSecurityInfo
from Products.CMFCore import CMFCorePermissions
from Products.Archetypes.public import BaseBTreeFolder, registerType
from Products.Archetypes.public import BaseBTreeFolderSchema
from Products.Archetypes.public import TemplateMixin, TemplateMixinSchema
from Products.TimeTracker.config import PROJECTNAME, NULL_DATE
from DateTime import DateTime
from StringIO import StringIO
import csv

schema = BaseBTreeFolderSchema.copy() + TemplateMixinSchema.copy()
layout = schema['layout'] 
layout.widget.visible = False

class TimeTracker(TemplateMixin, BaseBTreeFolder):
    security = ClassSecurityInfo()
    archetype_name = 'Time Tracker'
    schema = schema
    immediate_view = 'timetracker_tasks'
    default_view = 'timetracker_tasks'
    allowed_content_types = ('Task',)
    content_icon = 'timetracker.png'
    global_allow = 1
    filter_content_types = 0
    use_folder_tabs = 0
    _at_rename_after_creation = True

    __implements__ = BaseBTreeFolder.__implements__

    actions = TemplateMixin.actions + (
        { 'id':'tasks',
            'name':'Tasks',
            'action':'string:${object_url}/timetracker_tasks',
            'permissions':(CMFCorePermissions.AddPortalContent,),
        },
        { 'id':'download',
            'name':'Download',
            'action':'string:${object_url}/download_csv',
            'permissions':(CMFCorePermissions.View,),
        },
    )


    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'resumeTask')
    def saveTask(self, REQUEST=None):
        """
        Save a task's new field values after edited in the recent tasks view.
        """
        if REQUEST: 
            task_id = REQUEST.get('task_id')
            if task_id:
                ps = self.portal_catalog(id=task_id)
                if len(ps) == 1:
                    task = ps[0].getObject()
                    task.setTitle(self.REQUEST.get('task_input') or task.Title())
                    task.setActivity(self.REQUEST.get('act_input') or task.getActivity())
                    task.setCategory(self.REQUEST.get('cat_input') or task.getCategory())
                    msg = "Task saved succesfully"
                else:
                    msg = "Error: couldn't find task from the catalog."
            else:
                msg = "Error: couldn't find task id from request."

            url = REQUEST.URL1 + '/timetracker_tasks'
            REQUEST.set('portal_status_message',msg)
            REQUEST.RESPONSE.redirect(url)

    
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'resumeTask')
    def resumeTask(self, REQUEST=None):
        """ 
        Resume a task displayed in the 'Recent tasks' menu 
        """
        proxies = self.portal_catalog(getRunning=True, getSticky=False,
            path='/'.join(self.getPhysicalPath()))
        obs = [p.getObject() for p in proxies]
        for task in obs:
            task.stopTask()
            task.reindexObject()
        old_task = self._getOb(REQUEST.task_id)
        task_id = self.generateUniqueId('Task')
        self.invokeFactory(id = task_id, type_name='Task')
        task = self._getOb(task_id)
        task.setTitle(self.REQUEST.get('task_input') or  old_task.Title())
        task.setActivity(self.REQUEST.get('act_input') or old_task.getActivity())
        task.setCategory(self.REQUEST.get('cat_input') or old_task.getCategory())
        task.startTask(DateTime())
        if REQUEST: 
            url = REQUEST.URL1 + '/timetracker_tasks'
            REQUEST.RESPONSE.redirect(url)


    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'startTask')
    def startTask(self, REQUEST):
        """ 
        Start a pending task, i.e a task with a startDate in the future
        and displayed in the Pending Tasks section.
        """
        proxies = self.portal_catalog(getRunning=True, getSticky=False,
        path='/'.join(self.getPhysicalPath()))
        obs = [p.getObject() for p in proxies]
        for task in obs:
            task.stopTask()
        task = self._getOb(REQUEST.task_id)
        task.startTask(DateTime())
        url = REQUEST.URL1 + '/timetracker_tasks'
        REQUEST.RESPONSE.redirect(url)

        
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'getTimesheet')
    def getTimesheet(self, Category=None, REQUEST=None):
        """ 
        Export the task information to a spreadsheet 
        """
        category = self.REQUEST.get('category', None)
        date_select = self.REQUEST.get('date_select_radio', 'this_month')
        out = StringIO()
        writer = csv.writer(out) 
        total = 0
        
        if date_select == 'this_month':
            start_date = DateTime('%s %s 1' %(DateTime().year(), DateTime().month()))
            end_date = DateTime()
        elif date_select == 'previous_month':
            end_month = DateTime().month()
            end_year = DateTime().year()
            start_month = (end_month-1) or 12
            start_year = (start_month == 12) and end_year-1 or end_year
            start_date = DateTime('%s %s 1' % (start_year, start_month))
            end_date = DateTime('%s %s 1' %(end_month, end_year))
        elif date_select == 'custom':
            start_date = DateTime(self.REQUEST.get('start_date'))
            end_date = DateTime(self.REQUEST.get('end_date'))

        query = {
            'portal_type':'Task',
            'getStartDate':{'query':start_date, 'range':'min'},
            'getEndDate':{'query':end_date, 'range':'max'},
            'path':'/'.join(self.getPhysicalPath()),
            'sort_on':'getEndDate',
            'sort_order':'reverse',
            }

        if category:
            query['getCategory'] = category

        ps = self.portal_catalog(query)

        for t in ps:
            task = t.getObject()
            row = [
                task.Title(), 
                task.getActivity(), 
                task.getCategory(), 
                task.getStartTime(), 
                task.getEndTime(), 
                '%sh %sm' % task.timeSpent()
                ]
            total = total + task.getDuration()
            writer.writerow(row)

        time = int(total/60)
        total_hours = int(time/60)
        total_minutes = int(time%60)
        last_row = ['Total','','','','', '%sh %sm' % (total_hours, total_minutes)]
        writer.writerow(last_row)
        out.seek(0)
        out = out.read()
        if REQUEST:
            REQUEST.RESPONSE.setHeader('Content-Type',
                'text/comma-separated-values')
            REQUEST.RESPONSE.setHeader('Content-Disposition',
                'attachment; filename="Timesheet.csv"')
            REQUEST.RESPONSE.setHeader('Content-Length',str(len(out)))
        return out


def modify_fti(fti):
    d = {}
    for a in list(fti['actions']) + list(TimeTracker.actions):
        d[a['id']] = a
    # we need the 'view' action to keep Plone happy, we just make it invisible
    d['view']['visible'] = False
    alist = []
    # append actions in right order
    for action_id in ('view', 'tasks', 'download', 'edit'):
        alist.append(d[action_id])
    fti['actions'] = tuple(alist)
    return fti

registerType(TimeTracker, PROJECTNAME)
